/*
 * Copyright (c) 2014 Patrick P. Frey
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include "strus/base/dll_tags.hpp"
#include "strus/private/internationalization.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <libintl.h>
#include <locale.h>
#include <cstdarg>
#include <string>

#define STRUS_GETTEXT_PACKAGE		"strus-dom"
#define STRUS_GETTEXT_LOCALEDIR		""

std::runtime_error strus::runtime_error( const char* format, ...)
{
	char buffer[ 1024];
	va_list args;
	va_start( args, format);
	const char* formatTranslation = ::dgettext( STRUS_GETTEXT_PACKAGE, format);
	int buffersize = vsnprintf( buffer, sizeof(buffer), formatTranslation, args);
	buffer[ sizeof(buffer)-1] = 0;
	std::runtime_error rt( std::string( buffer, buffersize));
	va_end (args);
	return rt;
}

std::logic_error strus::logic_error( const char* format, ...)
{
	char buffer[ 1024];
	va_list args;
	va_start( args, format);
	const char* formatTranslation = ::dgettext( STRUS_GETTEXT_PACKAGE, format);
	int buffersize = vsnprintf( buffer, sizeof(buffer), formatTranslation, args);
	buffer[ sizeof(buffer)-1] = 0;
	std::logic_error rt( std::string( buffer, buffersize));
	va_end (args);
	return rt;
}

DLL_PUBLIC void strus::initMessageTextDomain()
{
#ifdef ENABLE_NLS
	::bindtextdomain( STRUS_GETTEXT_PACKAGE, STRUS_GETTEXT_LOCALEDIR);
#endif
}


